"use strict";
var firebase = require("firebase");
var rxjs_1 = require("rxjs");
var RxFirebaseAuth = (function () {
    function RxFirebaseAuth(app) {
        var _this = this;
        this.app = app;
        this.sdk = firebase.auth(this.app);
        this.onAuthStateChanged$ = new rxjs_1.Observable(function (subscriber) {
            _this.onAuthStateChanged$complete = _this.sdk.onAuthStateChanged(function (auth) { return subscriber.next(auth); });
        })
            .share()
            .observeOn(rxjs_1.Scheduler.asap);
        this.onSignIn$ = this.onAuthStateChanged$
            .filter(function (auth) { return !!auth; })
            .do(function (auth) { return _this.currentAuth = auth; })
            .share()
            .observeOn(rxjs_1.Scheduler.asap);
        var _a = this.onAuthStateChanged$
            .partition(function (auth) { return !auth && !!_this.currentAuth; }), signOut$ = _a[0], initiallySingedOut$ = _a[1];
        this.onSignOut$ = signOut$
            .map(function (auth) {
            var currentAuth = _this.currentAuth;
            _this.currentAuth = auth;
            return currentAuth;
        })
            .share()
            .observeOn(rxjs_1.Scheduler.asap);
        this.initiallySingedOut$ = initiallySingedOut$
            .do(function (auth) { return _this.currentAuth = auth; });
        this.isSignedIn$ = this.initiallySingedOut$
            .merge(this.onSignIn$)
            .merge(this.onSignOut$)
            .map(function () { return !!_this.currentAuth; })
            .publishBehavior(void 0)
            .refCount()
            .filter(function (isSignedIn) { return isSignedIn !== undefined; })
            .distinctUntilChanged();
    }
    Object.defineProperty(RxFirebaseAuth.prototype, "currentUser", {
        get: function () {
            return this.currentAuth;
        },
        enumerable: true,
        configurable: true
    });
    RxFirebaseAuth.prototype.createCustomToken = function (uid, developerClaims) {
        return this.sdk.createCustomToken(uid, developerClaims);
    };
    RxFirebaseAuth.prototype.applyActionCode = function (code) {
        return this.sdk.applyActionCode(code);
    };
    RxFirebaseAuth.prototype.checkActionCode = function (code) {
        return this.sdk.checkActionCode(code);
    };
    RxFirebaseAuth.prototype.confirmPasswordReset = function (code, newPassword) {
        return this.sdk.confirmPasswordReset(code, newPassword);
    };
    RxFirebaseAuth.prototype.createUserWithEmailAndPassword = function (email, password) {
        return this.sdk.createUserWithEmailAndPassword(email, password);
    };
    RxFirebaseAuth.prototype.fetchProvidersForEmail = function (email) {
        return this.sdk.fetchProvidersForEmail(email);
    };
    RxFirebaseAuth.prototype.getRedirectResult = function () {
        return this.sdk.getRedirectResult();
    };
    RxFirebaseAuth.prototype.sendPasswordResetEmail = function (email) {
        return this.sdk.sendPasswordResetEmail(email);
    };
    RxFirebaseAuth.prototype.signInAnonymously = function () {
        return this.sdk.signInAnonymously();
    };
    RxFirebaseAuth.prototype.signInWithCredential = function (credential) {
        return this.sdk.signInWithCredential(credential);
    };
    RxFirebaseAuth.prototype.signInWithCustomToken = function (token) {
        return this.sdk.signInWithCustomToken(token);
    };
    RxFirebaseAuth.prototype.signInWithEmailAndPassword = function (email, password) {
        return this.sdk.signInWithEmailAndPassword(email, password);
    };
    RxFirebaseAuth.prototype.signInWithPopup = function (provider) {
        return this.sdk.signInWithPopup(provider);
    };
    RxFirebaseAuth.prototype.signInWithRedirect = function (provider) {
        return this.sdk.signInWithRedirect(provider);
    };
    RxFirebaseAuth.prototype.signOut = function () {
        return this.sdk.signOut();
    };
    RxFirebaseAuth.prototype.verifyIdToken = function (idToken) {
        return this.sdk.verifyIdToken(idToken);
    };
    RxFirebaseAuth.prototype.verifyPasswordResetCode = function (code) {
        return this.sdk.verifyPasswordResetCode(code);
    };
    return RxFirebaseAuth;
}());
exports.RxFirebaseAuth = RxFirebaseAuth;
//# sourceMappingURL=auth.js.map