"use strict";
var rxjs_1 = require("rxjs");
var RxFirebase_1 = require("../RxFirebase");
var RxFirebaseDatabaseRef = (function () {
    function RxFirebaseDatabaseRef(db, from) {
        this.db = db;
        this.filters = { '_getSnapshot': true };
        if (from.url) {
            this.ref = this.db.sdk.refFromURL(from.url);
        }
        else if (from.path) {
            this.ref = this.db.sdk.ref(from.path);
        }
        else if (from.ref) {
            this.ref = from.ref;
        }
        else {
            throw new Error('RxFirebaseDatabaseRef: no reference specified');
        }
    }
    RxFirebaseDatabaseRef.applyFilter = function (observable, filter) {
        if (filter === 'untilSignIn') {
            return observable.takeUntil(RxFirebase_1.RxFirebase.auth.onSignIn$);
        }
        if (filter === 'untilSignOut') {
            return observable.takeUntil(RxFirebase_1.RxFirebase.auth.onSignOut$);
        }
        if (filter === 'afterSignIn') {
            return RxFirebase_1.RxFirebase.auth.isSignedIn$
                .switchMap(function (isSignedIn) { return isSignedIn
                ? observable
                : RxFirebase_1.RxFirebase.auth.onSignIn$.take(1).switchMap(function () { return observable; }); });
        }
        if (filter === 'afterSignOut') {
            return RxFirebase_1.RxFirebase.auth.onSignOut$.take(1).switchMap(function () { return observable; });
        }
        if (filter === '_getSnapshot') {
            return observable.map(function (_a) {
                var snapshot = _a.snapshot;
                return snapshot;
            });
        }
        if (filter === 'getValue') {
            return observable.map(function (snapshot) { return snapshot.val(); });
        }
        if (filter === 'asList') {
            return observable.map(function (value) {
                var array = [];
                for (var key in value) {
                    if (value.hasOwnProperty(key)) {
                        var element = value[key];
                        if (typeof element !== 'object') {
                            element = { $value: element };
                        }
                        element.$key = key;
                        array.push(element);
                    }
                }
                return array;
            });
        }
        return observable;
    };
    RxFirebaseDatabaseRef.prototype.child = function (path) {
        this.ref = this.ref.child(path);
        return this;
    };
    Object.defineProperty(RxFirebaseDatabaseRef.prototype, "parent", {
        get: function () {
            this.ref = this.ref.parent;
            return this;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RxFirebaseDatabaseRef.prototype, "root", {
        get: function () {
            this.ref = this.ref.root;
            return this;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RxFirebaseDatabaseRef.prototype, "key", {
        get: function () {
            return this.ref.key;
        },
        enumerable: true,
        configurable: true
    });
    RxFirebaseDatabaseRef.prototype.untilSignIn = function () {
        this.filters['untilSignIn'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.untilSignOut = function () {
        this.filters['untilSignOut'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.afterSignIn = function () {
        this.filters['afterSignIn'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.afterSignOut = function () {
        this.filters['afterSignOut'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.getValue = function () {
        this.filters['getValue'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.withPrevChildKey = function () {
        this.filters['_getSnapshot'] = false;
        this.filters['withPrevChildKey'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.asList = function () {
        this.filters['getValue'] = true;
        this.filters['asList'] = true;
        return this;
    };
    RxFirebaseDatabaseRef.prototype.onValue = function () {
        return this.on('value');
    };
    RxFirebaseDatabaseRef.prototype.onChildAdded = function () {
        return this.on('child_added');
    };
    RxFirebaseDatabaseRef.prototype.onChildRemoved = function () {
        return this.on('child_removed');
    };
    RxFirebaseDatabaseRef.prototype.onChildChanged = function () {
        return this.on('child_changed');
    };
    RxFirebaseDatabaseRef.prototype.onChildMoved = function () {
        return this.on('child_moved');
    };
    RxFirebaseDatabaseRef.prototype.onceValue = function () {
        return this.once('value');
    };
    RxFirebaseDatabaseRef.prototype.onceChildAdded = function () {
        return this.once('child_added');
    };
    RxFirebaseDatabaseRef.prototype.onceChildRemoved = function () {
        return this.once('child_removed');
    };
    RxFirebaseDatabaseRef.prototype.onceChildChanged = function () {
        return this.once('child_changed');
    };
    RxFirebaseDatabaseRef.prototype.onceChildMoved = function () {
        return this.once('child_moved');
    };
    RxFirebaseDatabaseRef.prototype.startAt = function (value, key) {
        this.ref = this.ref.startAt(value, key);
        return this;
    };
    RxFirebaseDatabaseRef.prototype.endAt = function (value, key) {
        this.ref = this.ref.endAt(value, key);
        return this;
    };
    RxFirebaseDatabaseRef.prototype.equalTo = function (value, key) {
        this.ref = this.ref.equalTo(value, key);
        return this;
    };
    RxFirebaseDatabaseRef.prototype.isEqual = function (other) {
        return this.ref.isEqual(other && other.ref);
    };
    RxFirebaseDatabaseRef.prototype.limitToFirst = function (limit) {
        this.ref = this.ref.limitToFirst(limit);
        return this;
    };
    RxFirebaseDatabaseRef.prototype.limitToLast = function (limit) {
        this.ref = this.ref.limitToLast(limit);
        return this;
    };
    RxFirebaseDatabaseRef.prototype.orderByChild = function (path) {
        this.ref = this.ref.orderByChild(path);
        return this;
    };
    RxFirebaseDatabaseRef.prototype.orderByKey = function () {
        this.ref = this.ref.orderByKey();
        return this;
    };
    RxFirebaseDatabaseRef.prototype.orderByPriority = function () {
        this.ref = this.ref.orderByPriority();
        return this;
    };
    RxFirebaseDatabaseRef.prototype.orderByValue = function () {
        this.ref = this.ref.orderByValue();
        return this;
    };
    RxFirebaseDatabaseRef.prototype.toString = function () {
        return this.ref.toString();
    };
    RxFirebaseDatabaseRef.prototype.on = function (eventType) {
        var _this = this;
        var observable = new rxjs_1.Observable(function (subscriber) {
            _this.callback = function (snapshot, prevChildKey) {
                subscriber.next({ snapshot: snapshot, prevChildKey: prevChildKey });
            };
            _this.ref.on(eventType, _this.callback, function (err) { return subscriber.error(err); });
            return function () { return _this.ref.off(eventType, _this.callback); };
        });
        return this.applyFilters(observable);
    };
    RxFirebaseDatabaseRef.prototype.once = function (eventType) {
        var _this = this;
        var observable = new rxjs_1.Observable(function (subscriber) {
            _this.callback = function (snapshot, prevChildKey) {
                subscriber.next({ snapshot: snapshot, prevChildKey: prevChildKey });
                subscriber.complete();
            };
            _this.ref.once(eventType, _this.callback, function (err) { return subscriber.error(err); });
            return function () { return _this.ref.off(eventType, _this.callback); };
        });
        return this.applyFilters(observable);
    };
    RxFirebaseDatabaseRef.prototype.onDisconnect = function () {
        return this.ref.onDisconnect();
    };
    RxFirebaseDatabaseRef.prototype.push = function (value, onComplete) {
        return new RxFirebaseDatabaseRef(this.db, { ref: this.ref.push(value, onComplete) });
    };
    RxFirebaseDatabaseRef.prototype.remove = function (onComplete) {
        return this.ref.remove(onComplete);
    };
    RxFirebaseDatabaseRef.prototype.set = function (value, onComplete) {
        return this.ref.set(value, onComplete);
    };
    RxFirebaseDatabaseRef.prototype.setPriority = function (priority, onComplete) {
        return this.ref.setPriority(priority, onComplete);
    };
    RxFirebaseDatabaseRef.prototype.setWithPriority = function (newVal, newPriority, onComplete) {
        return this.ref.setWithPriority(newVal, newPriority, onComplete);
    };
    RxFirebaseDatabaseRef.prototype.transaction = function (transactionUpdate, onComplete, applyLocally) {
        return this.ref.transaction(transactionUpdate, onComplete, applyLocally);
    };
    RxFirebaseDatabaseRef.prototype.update = function (values, onComplete) {
        return this.ref.update(values, onComplete);
    };
    RxFirebaseDatabaseRef.prototype.applyFilters = function (observable) {
        if (this.filters['withPrevChildKey'] && this.filters['getValue']) {
            throw new Error('RxFirebaseDatabaseRef: withPrevChildKey() cannot be used in conjunction with getValue() or asList()');
        }
        for (var filter in this.filters) {
            observable = RxFirebaseDatabaseRef.applyFilter(observable, filter);
        }
        return observable;
    };
    return RxFirebaseDatabaseRef;
}());
exports.RxFirebaseDatabaseRef = RxFirebaseDatabaseRef;
//# sourceMappingURL=ref.js.map